@@ -1,6 +1,6 @@
 import sys, argparse
 
-from .wiretrace import WireTrace
+from .storage import WireTrace
 from .visualizer import Visualizer
 
 def main():
@@ -16,6 +16,9 @@ def main():
     parser.add_argument('-e', '--end', 
         type=str, metavar='FORMULA', dest='end', 
         help='formula for the end of the window')
+    parser.add_argument('-b', '--break',
+        type=str, metavar='FORMULA', dest='breakpoints',
+        help='formula for the points in time to be highlighted')
     parser.add_argument('-l', '--length', 
         type=int, dest='length', 
         help='number of cycles to display')
@@ -48,14 +51,19 @@ def main():
             start = 0
         length = args.length if args.length is not None else wiretrace.length() - start
     
-    wires = set()
+    # Calculate breakpoints
+    breakpoints = None
+    if args.breakpoints is not None:
+        breakpoints = wiretrace.evaluate(args.breakpoints)
+
+    wires = None
     if args.wires:
-        wires = set(args.wires.split(','))
+        wires = set([name.strip() for name in args.wires.split(',')])
     
     # Convert wiretrace to graphical vector image.
-    image = Visualizer().to_svg(wiretrace, start=start, length=length, wires=wires)
+    image = Visualizer().to_svg(wiretrace, start=start, length=length, wires=wires, breakpoints=breakpoints)
 
-    if len(wires):
+    if wires is not None and len(wires):
         raise Exception(f'Unknown wires {wires.__repr__()}\nThe following wires were detected in the wiretrace:\n{wiretrace.get_wire_names()}')
     
     if args.display:
