@@ -11,8 +11,10 @@ class Style:
     class Default():
         TOP_MARGIN = 15
         LEFT_MARGIN = 15
-        TEXT_WIDTH = 100
-        DATA_WIDTH = 50
+        CHAR_WIDTH = 7
+        LABEL_WIDTH = 12
+        TEXT_WIDTH = CHAR_WIDTH * LABEL_WIDTH
+        FULL_WIDTH = 800
         WIRE_HEIGHT = 20
         WIRE_MARGIN = 10
         TRANS_START = 5
@@ -24,6 +26,7 @@ class Style:
         LINE_COLOR_Z = "#FFFF00"
         LINE_COLOR_X = "#FF0000"
         LINE_COLOR_DATA = "#3DB8B8"
+        BREAKPOINT_COLOR = "#FFFF00"
         TEXT_COLOR = "#FFFFFF"
         BKGD_COLOR = "#000000"
         #wires going from 0 and 1 are two different colors, x variable is red rectangle, z variable is yellow
@@ -51,6 +54,9 @@ class Style:
         #BKGD_COLOR = "#003000"
         BKGD_COLOR = "#2b5e2b"
 
+    class Debug(Default):
+        pass
+
 
 class Visualizer:
     """Converter for wiretrace objects to a svg vector image format."""
@@ -59,44 +65,78 @@ class Visualizer:
         """Optionally pass in a style class to control how the visualizer looks."""
         self.style = style
 
-    def to_svg(self, wiretrace, start=0, length=None, wires=set()):
+    def to_svg(self, wiretrace, start=0, length=None, wires=None, breakpoints=None):
         if length is None:
             length = wiretrace.length()
         """Converts the provided wiretrace object to a VectorImage object (svg)."""
-        return VectorImage(self._wiretrace_to_svg(wiretrace, start, length, wires))
+        return VectorImage(self._wiretrace_to_svg(wiretrace, start, length, wires, breakpoints))
 
-    def _wiretrace_to_svg(self, wiretrace, start, length, wires=None):
-        width = 2 * self.style.LEFT_MARGIN + self.style.TEXT_WIDTH + length * self.style.DATA_WIDTH
-        height = 2 * self.style.TOP_MARGIN + (len(wires) + 1) * (self.style.WIRE_HEIGHT + self.style.WIRE_MARGIN) - self.style.WIRE_MARGIN
-        if len(wires) == 0:
+    def _wiretrace_to_svg(self, wiretrace, start, length, wires=None, breakpoints=None):
+        if wires and len(wires) == 0:  # include all wires if empty list provided
             wires = None
-            height += sum([sum([1 for wire in wiregroup.wires]) for wiregroup in wiretrace.wiregroups]) * (self.style.WIRE_HEIGHT + self.style.WIRE_MARGIN)
-        
+        width = 2 * self.style.LEFT_MARGIN + self.style.TEXT_WIDTH + self.style.FULL_WIDTH
+        height = 2 * self.style.TOP_MARGIN - self.style.WIRE_MARGIN + (self.style.WIRE_HEIGHT + \
+            self.style.WIRE_MARGIN) * (1 + (len(wires) if wires else wiretrace.num_wires()))
+
         svg = f'<svg viewBox="0 0 {width} {height}" xmlns="http://www.w3.org/2000/svg">' \
               f'<rect x="0" y="0" width="{width}" height="{height}" fill="{self.style.BKGD_COLOR}" />'
         
-        lines = 1
         svg += self._timestamps_to_svg(
             left = self.style.LEFT_MARGIN + self.style.TEXT_WIDTH, 
             top = self.style.TOP_MARGIN,
             start = start,
             length = length)
+
+        if not breakpoints:
+            breakpoints = []
+        svg += self._breakpoints_to_svg(
+            breakpoints, 
+            left = self.style.LEFT_MARGIN + self.style.TEXT_WIDTH, 
+            top = self.style.TOP_MARGIN, 
+            start = start,
+            length = length,
+            height = height - 2 * self.style.TOP_MARGIN + self.style.WIRE_MARGIN)
+        
+        svg += self._wiregroup_to_svg(
+            wiregroup = wiretrace.root,
+            left = self.style.LEFT_MARGIN,
+            top = self.style.TOP_MARGIN + self.style.WIRE_HEIGHT + self.style.WIRE_MARGIN, 
+            start = start,
+            length = length,
+            wires = wires)[0]  # the first element is the svg data
         
-        for wiregroup in wiretrace.wiregroups:
-            svg += self._wiregroup_to_svg(
-                wiregroup = wiregroup, 
-                left = self.style.LEFT_MARGIN,
-                top = self.style.TOP_MARGIN + self.style.WIRE_HEIGHT + self.style.WIRE_MARGIN,
-                start = start,
-                length = length,
-                wires = wires)
         svg += '</svg>'
         return svg
 
     def _timestamps_to_svg(self, left, top, start, length):
         svg = ''
-        for index in range(start, start + length):
-            svg += f'<text x="{left + (index - start + 1/2) * self.style.DATA_WIDTH}" y="{top + (self.style.WIRE_HEIGHT + self.style.WIRE_MARGIN) / 2}" class="small" fill="{self.style.TEXT_COLOR}" text-anchor="middle">{index}</text>'
+        for index in range(start, start + length, ((length - 1) // 32) + 1):
+            svg += self._shape_to_svg({
+                'name': 'text',
+                'x': left + (index - start + 1/2) * (self.style.FULL_WIDTH / length),
+                'y': top + (self.style.WIRE_HEIGHT + self.style.WIRE_MARGIN) / 2,
+                'class': 'small',
+                'fill': self.style.TEXT_COLOR,
+                'text-anchor': 'middle',
+                'font-family': 'monospace',
+                'content': index,
+            })
+        return svg
+
+    def _breakpoints_to_svg(self, breakpoints, left, top, start, length, height):
+        """Convert a list of breakpoint times to highlights on the svg."""
+        svg = ''
+        for index in breakpoints:
+            if index >= start and index < start + length:
+                svg += self._shape_to_svg({
+                    'name': 'rect',
+                    'x': left + (index - start) * (self.style.FULL_WIDTH / length) + self.style.TRANS_START,
+                    'y': top,
+                    'width': (self.style.FULL_WIDTH / length),
+                    'height': height,
+                    'fill': self.style.BREAKPOINT_COLOR,
+                    'fill-opacity': 0.4,
+                })
         return svg
 
     def _wiregroup_to_svg(self, wiregroup, left, top, start, length, wires=None):
@@ -104,28 +144,46 @@ class Visualizer:
         index = 0
         for wire in wiregroup.wires:
             if wires == None or wire.name in wires:
+                if wires:  # ensure only one copy of the wire is included
+                    wires.remove(wire.name)
                 svg += self._wire_to_svg(
                     wire,
                     left = left,
                     top = top + (index * (self.style.WIRE_HEIGHT + self.style.WIRE_MARGIN)),
                     start = start,
                     length = length)
-                if wires:
-                    wires.remove(wire.name)
                 index += 1
-        return svg
+        # recursively call function on nested wiregroups
+        for group in wiregroup.groups:
+            result = self._wiregroup_to_svg(
+                group,
+                left = left,
+                top = top + (index * (self.style.WIRE_HEIGHT + self.style.WIRE_MARGIN)),
+                start = start,
+                length = length,
+                wires = wires)
+            svg += result[0]
+            index += result[1]
+        return svg, index
 
     def _wire_to_svg(self, wire, left, top, start, length):
-        svg = f'<text x="{left}" y="{top + 15}" class="small" fill="{self.style.TEXT_COLOR}">{wire.name}</text>'
+        svg = self._shape_to_svg({
+            'name': 'text',
+            'x': left,
+            'y': top + 15,
+            'class': 'small',
+            'fill': self.style.TEXT_COLOR,
+            'font-family': 'monospace',
+            'content': wire.name if len(wire.name) <= 10 else wire.name[:7] + '...',
+        })
         for index in range(start, start + length):
-            prev = (wire.data[index - 1] if index > 0 else wire.data[index]) if index < len(wire.data) else wire.data[-1]
-            value = wire.data[index] if index < len(wire.data) else wire.data[-1]
             svg += self._value_to_svg(
-                prev = prev,
-                value = value,
-                width = wire.width,
-                left = left + ((index - start) * self.style.DATA_WIDTH) + self.style.TEXT_WIDTH,
+                prev = wire[index - 1] if index > 0 else wire[index],
+                value = wire[index],
+                width = wire.width(),
+                left = left + ((index - start) * (self.style.FULL_WIDTH / length)) + self.style.TEXT_WIDTH,
                 top = top,
+                length = length,
                 initial = (index == start))
         return svg
 
@@ -147,6 +205,8 @@ class Visualizer:
                 return Visualizer.ValueType.X
             elif value in ('z', 'Z'):
                 return Visualizer.ValueType.Z
+            elif value is None:
+                return Visualizer.ValueType.X
             else:
                 raise SoottyInternalError(f'Invalid wire value, unable to visualize: {value}')
         else:
@@ -155,7 +215,7 @@ class Visualizer:
             else:
                 return Visualizer.ValueType.DATA
 
-    def _value_to_svg(self, prev, value, width, left, top, initial=False):
+    def _value_to_svg(self, prev, value, width, left, top, length, initial=False):
         # deduce types from wire width and value:
         prev_type = Visualizer.type_from_value(prev, width)
         value_type = Visualizer.type_from_value(value, width)
@@ -168,7 +228,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_LOW,
@@ -193,7 +253,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left + self.style.TRANS_START + self.style.TRANS_WIDTH * self.style.BLOCK_TRANS,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top,
                 'y2': top,
                 'stroke': self.style.LINE_COLOR_HIGH,
@@ -218,7 +278,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left + self.style.TRANS_START + self.style.TRANS_WIDTH * self.style.BLOCK_TRANS,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_LOW,
@@ -227,7 +287,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top,
                 'y2': top,
                 'stroke': self.style.LINE_COLOR_HIGH,
@@ -236,7 +296,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_DATA,
@@ -252,7 +312,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left + self.style.TRANS_START + self.style.TRANS_WIDTH,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top,
                 'y2': top,
                 'stroke': self.style.LINE_COLOR_DATA,
@@ -261,7 +321,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top,
                 'y2': top,
                 'stroke': self.style.LINE_COLOR_DATA,
@@ -277,7 +337,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left + self.style.TRANS_START + self.style.TRANS_WIDTH,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_DATA,
@@ -310,7 +370,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left + self.style.TRANS_START + self.style.TRANS_WIDTH,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_LOW,
@@ -343,7 +403,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left + self.style.TRANS_START + self.style.TRANS_WIDTH,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top,
                 'y2': top,
                 'stroke': self.style.LINE_COLOR_HIGH,
@@ -352,7 +412,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_DATA,
@@ -360,7 +420,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top,
                 'y2': top,
                 'stroke': self.style.LINE_COLOR_DATA,
@@ -385,7 +445,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left + self.style.TRANS_START + self.style.TRANS_WIDTH,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_DATA,
@@ -409,7 +469,7 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left + self.style.TRANS_START + self.style.TRANS_WIDTH,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top,
                 'y2': top,
                 'stroke': self.style.LINE_COLOR_DATA,
@@ -420,6 +480,7 @@ class Visualizer:
                 'y': top + (self.style.WIRE_HEIGHT + self.style.WIRE_MARGIN) / 2,
                 'class': 'small',
                 'fill': self.style.TEXT_COLOR,
+                'font-family': 'monospace',
                 'content': value
             })
         # TODO: figure out how to display X values in svg:
@@ -427,161 +488,176 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + self.style.TRANS_START,
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
-                'stroke': self.style.LINE_COLOR_X,
+                'stroke': self.style.LINE_COLOR_LOW,
             })
             shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
+                'name': 'rect',
+                'x': left,
+                'y': top,
+                'width': (self.style.FULL_WIDTH / length),
+                'height': self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_X,
+                'fill': self.style.LINE_COLOR_X,
             })
         elif prev_type is Visualizer.ValueType.HIGH and value_type is Visualizer.ValueType.X:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
-                'stroke': self.style.LINE_COLOR_X,
-            })
-            shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + self.style.TRANS_START,
                 'y1': top,
                 'y2': top,
+                'stroke': self.style.LINE_COLOR_HIGH,
+            })
+            shapes.append({
+                'name': 'rect',
+                'x': left,
+                'y': top,
+                'width': (self.style.FULL_WIDTH / length),
+                'height': self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_X,
+                'fill': self.style.LINE_COLOR_X,
             })
         elif prev_type is Visualizer.ValueType.DATA and value_type is Visualizer.ValueType.X:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_X,
             })
             shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
+                'name': 'rect',
+                'x': left,
+                'y': top,
+                'width': (self.style.FULL_WIDTH / length),
+                'height': self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_X,
+                'fill': self.style.LINE_COLOR_X,
             })
         elif prev_type is Visualizer.ValueType.X and value_type is Visualizer.ValueType.LOW:
             shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
+                'name': 'rect',
+                'x': left,
+                'y': top,
+                'width': self.style.TRANS_START,
+                'height': self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_X,
+                'fill': self.style.LINE_COLOR_X,
             })
             shapes.append({
                 'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
-                'stroke': self.style.LINE_COLOR_X,
+                'x1': left + self.style.TRANS_START,
+                'x2': left + (self.style.FULL_WIDTH / length),
+                'y1': top + self.style.WIRE_HEIGHT,
+                'y2': top + self.style.WIRE_HEIGHT,
+                'stroke': self.style.LINE_COLOR_LOW,
             })
         elif prev_type is Visualizer.ValueType.X and value_type is Visualizer.ValueType.HIGH:
             shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
+                'name': 'rect',
+                'x': left,
+                'y': top,
+                'width': self.style.TRANS_START,
+                'height': self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_X,
+                'fill': self.style.LINE_COLOR_X,
             })
             shapes.append({
                 'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x1': left + self.style.TRANS_START,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top,
                 'y2': top,
-                'stroke': self.style.LINE_COLOR_X,
+                'stroke': self.style.LINE_COLOR_HIGH,
             })
         elif prev_type is Visualizer.ValueType.X and value_type is Visualizer.ValueType.DATA:
             shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
+                'name': 'rect',
+                'x': left,
+                'y': top,
+                'width': (self.style.FULL_WIDTH / length),
+                'height': self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_X,
+                'fill': self.style.LINE_COLOR_X,
             })
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top,
                 'y2': top,
                 'stroke': self.style.LINE_COLOR_X,
             })
         elif prev_type is Visualizer.ValueType.X and value_type is Visualizer.ValueType.X:
             shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
-                'stroke': self.style.LINE_COLOR_X,
-            })
-            shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
+                'name': 'rect',
+                'x': left,
+                'y': top,
+                'width': (self.style.FULL_WIDTH / length),
+                'height': self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_X,
+                'fill': self.style.LINE_COLOR_X,
             })
         # TODO: figure out how to display Z values in svg:
         elif prev_type is Visualizer.ValueType.LOW and value_type is Visualizer.ValueType.Z:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + self.style.TRANS_START,
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
+                'stroke': self.style.LINE_COLOR_LOW,
+            })
+            shapes.append({
+                'name': 'line',
+                'x1': left + self.style.TRANS_START,
+                'x2': left + self.style.TRANS_START + self.style.TRANS_WIDTH * self.style.BLOCK_TRANS,
+                'y1': top + self.style.WIRE_HEIGHT,
+                'y2': (top + (self.style.WIRE_HEIGHT)/2),
                 'stroke': self.style.LINE_COLOR_Z,
             })
             shapes.append({
                 'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
+                'x1': left + self.style.TRANS_START,
+                'x2': left + (self.style.FULL_WIDTH / length),
+                'y1': (top + (self.style.WIRE_HEIGHT)/2),
+                'y2': (top + (self.style.WIRE_HEIGHT)/2),
                 'stroke': self.style.LINE_COLOR_Z,
             })
         elif prev_type is Visualizer.ValueType.HIGH and value_type is Visualizer.ValueType.Z:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
+                'x2': left + self.style.TRANS_START,
+                'y1': top,
+                'y2': top,
+                'stroke': self.style.LINE_COLOR_HIGH,
+            })
+            shapes.append({
+                'name': 'line',
+                'x1': left + self.style.TRANS_START,
+                'x2': left + self.style.TRANS_START + self.style.TRANS_WIDTH * self.style.BLOCK_TRANS,
+                'y1': top,
+                'y2': (top + (self.style.WIRE_HEIGHT)/2),
                 'stroke': self.style.LINE_COLOR_Z,
             })
             shapes.append({
                 'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
+                'x1': left + self.style.TRANS_START + self.style.TRANS_WIDTH * self.style.BLOCK_TRANS,
+                'x2': left + (self.style.FULL_WIDTH / length),
+                'y1': (top + (self.style.WIRE_HEIGHT)/2),
+                'y2': (top + (self.style.WIRE_HEIGHT)/2),
                 'stroke': self.style.LINE_COLOR_Z,
             })
         elif prev_type is Visualizer.ValueType.DATA and value_type is Visualizer.ValueType.Z:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top + self.style.WIRE_HEIGHT,
                 'y2': top + self.style.WIRE_HEIGHT,
                 'stroke': self.style.LINE_COLOR_X,
@@ -589,111 +665,104 @@ class Visualizer:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
-                'stroke': self.style.LINE_COLOR_X,
+                'x2': left + (self.style.FULL_WIDTH / length),
+                'y1': (top + (self.style.WIRE_HEIGHT)/2),
+                'y2': (top + (self.style.WIRE_HEIGHT)/2),
+                'stroke': self.style.LINE_COLOR_Z,
             })
         elif prev_type is Visualizer.ValueType.X and value_type is Visualizer.ValueType.Z:
             shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
-                'stroke': self.style.LINE_COLOR_Z,
+                'name': 'rect',
+                'x': left,
+                'y': top,
+                'width': self.style.TRANS_START,
+                'height': self.style.WIRE_HEIGHT,
+                'stroke': self.style.LINE_COLOR_X,
+                'fill': self.style.LINE_COLOR_X,
             })
             shapes.append({
                 'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
+                'x1': left + self.style.TRANS_START,
+                'x2': left + (self.style.FULL_WIDTH / length),
+                'y1': (top + (self.style.WIRE_HEIGHT)/2),
+                'y2': (top + (self.style.WIRE_HEIGHT)/2),
                 'stroke': self.style.LINE_COLOR_Z,
             })
         elif prev_type is Visualizer.ValueType.Z and value_type is Visualizer.ValueType.LOW:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
+                'x2': left + self.style.TRANS_START,
+                'y1': (top + (self.style.WIRE_HEIGHT)/2),
+                'y2': (top + (self.style.WIRE_HEIGHT)/2),
                 'stroke': self.style.LINE_COLOR_Z,
             })
             shapes.append({
                 'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
-                'stroke': self.style.LINE_COLOR_Z,
+                'x1': left + self.style.TRANS_START,
+                'x2': left + self.style.TRANS_START + self.style.TRANS_WIDTH * self.style.BLOCK_TRANS,
+                'y1': (top + (self.style.WIRE_HEIGHT)/2),
+                'y2': top + self.style.WIRE_HEIGHT,
+                'stroke': self.style.LINE_COLOR_LOW,
+            })
+            shapes.append({
+                'name': 'line',
+                'x1': left + self.style.TRANS_START + self.style.TRANS_WIDTH * self.style.BLOCK_TRANS,
+                'x2': left + (self.style.FULL_WIDTH / length),
+                'y1': top + self.style.WIRE_HEIGHT,
+                'y2': top + self.style.WIRE_HEIGHT,
+                'stroke': self.style.LINE_COLOR_LOW,
             })
         elif prev_type is Visualizer.ValueType.Z and value_type is Visualizer.ValueType.HIGH:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
+                'x2': left + self.style.TRANS_START,
+                'y1': (top + (self.style.WIRE_HEIGHT)/2),
+                'y2': (top + (self.style.WIRE_HEIGHT)/2),
                 'stroke': self.style.LINE_COLOR_Z,
             })
             shapes.append({
                 'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
+                'x1': left + self.style.TRANS_START,
+                'x2': left + self.style.TRANS_START + self.style.TRANS_WIDTH * self.style.BLOCK_TRANS,
+                'y1': (top + (self.style.WIRE_HEIGHT)/2),
+                'y2': top,
+                'stroke': self.style.LINE_COLOR_HIGH,
+            })
+            shapes.append({
+                'name': 'line',
+                'x1': left + self.style.TRANS_START,
+                'x2': left + (self.style.FULL_WIDTH / length),
                 'y1': top,
                 'y2': top,
-                'stroke': self.style.LINE_COLOR_Z,
-            })
-        elif prev_type is Visualizer.ValueType.Z and value_type is Visualizer.ValueType.DATA:
-            shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
-                'stroke': self.style.LINE_COLOR_Z,
-            })
-            shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
-                'stroke': self.style.LINE_COLOR_Z,
+                'stroke': self.style.LINE_COLOR_HIGH,
             })
         elif prev_type is Visualizer.ValueType.Z and value_type is Visualizer.ValueType.X:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
+                'x2': left + (self.style.FULL_WIDTH / length),
+                'y1': (top + (self.style.WIRE_HEIGHT)/2),
+                'y2': (top + (self.style.WIRE_HEIGHT)/2),
                 'stroke': self.style.LINE_COLOR_Z,
             })
             shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
-                'stroke': self.style.LINE_COLOR_Z,
+                'name': 'rect',
+                'x': left,
+                'y': top,
+                'width': (self.style.FULL_WIDTH / length),
+                'height': self.style.WIRE_HEIGHT,
+                'stroke': self.style.LINE_COLOR_X,
+                'fill': self.style.LINE_COLOR_X,
             })
         elif prev_type is Visualizer.ValueType.Z and value_type is Visualizer.ValueType.Z:
             shapes.append({
                 'name': 'line',
                 'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top + self.style.WIRE_HEIGHT,
-                'y2': top + self.style.WIRE_HEIGHT,
-                'stroke': self.style.LINE_COLOR_Z,
-            })
-            shapes.append({
-                'name': 'line',
-                'x1': left,
-                'x2': left + self.style.DATA_WIDTH,
-                'y1': top,
-                'y2': top,
+                'x2': left + (self.style.FULL_WIDTH / length),
+                'y1': (top + (self.style.WIRE_HEIGHT)/2),
+                'y2': (top + (self.style.WIRE_HEIGHT)/2),
                 'stroke': self.style.LINE_COLOR_Z,
             })
         else:
@@ -701,12 +770,16 @@ class Visualizer:
         
         svg_data = ''
         for shape in shapes:
-            start_tag = '<' + str(shape['name'])
-            end_tag = ' />'
-            for prop in shape:
-                if prop == 'content':
-                    end_tag = '>' + str(shape['content']) + '</' + str(shape['name']) + '>'
-                elif prop != 'name':
-                    start_tag += ' ' + prop + '="' + str(shape[prop]) + '"'
-            svg_data += start_tag + end_tag
+            svg_data += self._shape_to_svg(shape)
         return svg_data
+
+    def _shape_to_svg(self, shape):
+        """Convert a shape dictionary object to an svg string."""
+        start_tag = '<' + str(shape['name'])
+        end_tag = ' />'
+        for prop in shape:
+            if prop == 'content':
+                end_tag = '>' + str(shape['content']) + '</' + str(shape['name']) + '>'
+            elif prop != 'name':
+                start_tag += ' ' + prop + '="' + str(shape[prop]) + '"'
+        return start_tag + end_tag
